from typing import Any, Dict
from setuptools import Extension


def build(setup_kwargs: Dict[str, Any]) -> None:
    print("Inside the Build method")
    setup_kwargs.update(
        {
            "zip_safe": False,
            "build_golang": {"root": "github.com/10in30/loophost"},
            "ext_modules": [
                Extension(
                    "loophost",
                    ["loophost/main.go"],
                    ["loophost/reverseproxy"],
                    py_limited_api=True,
                    define_macros=[("Py_LIMITED_API", None)],
                )
            ],
            "setup_requires": ["setuptools-golang"],
        }
    )



import json
import os
import sys
import fling_hub
import pathlib
import subprocess
from subprocess import run, call, Popen
from fling_cli.auth import gh_authenticate
import shutil
from string import Template


TARGET_DIR = pathlib.Path(pathlib.Path.home(), ".flingdev")
PYEX = sys.executable
HUBDIR = os.path.dirname(os.path.realpath(fling_hub.__file__))
USERNAME = None


def restart_as_sudo():
    print(
        "Switching to root user to install web services (you will be prompted for your password)"
    )
    run(
        "sudo python3 -m fling_hub.postinstall",
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        stdin=subprocess.PIPE,
        cwd=TARGET_DIR,
    )
    print("All finished.")
    sys.exit()


def post_install_one():
    print("Installing LoopHost...")
    os.makedirs(pathlib.Path(TARGET_DIR, "certs"), exist_ok=True)
    os.chdir(TARGET_DIR)
    authenticate_with_fling()
    issue_certs()
    restart_as_sudo()


def post_install_two():
    setup_launchd_scripts()


def authenticate_with_fling():
    global USERNAME
    gh_authenticate()
    if not os.path.exists(pathlib.Path(TARGET_DIR, "flinguser.txt")):
        raise Exception("Github authentication failed, can't continue install.")
    with open(pathlib.Path(TARGET_DIR, "flinguser.txt"), "r") as userfile:
        USERNAME = userfile.read()


def issue_certs():
    global USERNAME, TARGET_DIR
    print("Generating SSL certificates (this may take a minute)...")
    run(
        " ".join(
            [
                "certbot",
                "certonly",
                "--config-dir ./",
                "--work-dir ./",
                "--logs-dir ./",
                "--non-interactive",
                "--expand",
                "--agree-tos",
                "-m webmaster@fling.dev",
                "--authenticator=fling_authenticator",
                f'-d "*.{USERNAME}.fling.dev"',
                f'-d "{USERNAME}.fling.dev"',
                "--fling_authenticator-propagation-seconds=15",
            ]
        ),
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        stdin=subprocess.PIPE,
        cwd=TARGET_DIR,
        shell=True,
        check=True,
    )


def setup_launchd_scripts():
    global USERNAME, TARGET_DIR, HUBDIR, PYEX
    if not os.path.exists(pathlib.Path(TARGET_DIR, "loophost.json")):
        with open(pathlib.Path(TARGET_DIR, "loophost.json"), "w") as datafile:
            data = {"apps": {}, "fqdn": f"{USERNAME}.fling.dev"}
            datafile.write(json.dumps(data))
    shutil.copy2(pathlib.Path(HUBDIR, "loophost.plist.template"), TARGET_DIR)
    shutil.copy2(pathlib.Path(HUBDIR, "hub.plist.template"), TARGET_DIR)

    d = {"CWD": TARGET_DIR, "USERNAME": USERNAME, "PYEX": PYEX, "HUBDIR": HUBDIR}

    templates = [
        (
            pathlib.Path(HUBDIR, "loophost.plist.template"),
            "/Library/LaunchDaemons/dev.fling.hub.local.plist",
        ),
        (
            pathlib.Path(HUBDIR, "hub.plist.template"),
            "/Library/LaunchDaemons/dev.fling.hub.plist",
        ),
    ]

    for infile, outfile in templates:
        with open(infile, "r") as f:
            src = Template(f.read())
            result = src.substitute(d)
            with open(outfile, "w") as o:
                o.write(result)

            run(["launchctl", "unload", outfile], cwd=TARGET_DIR, user="root")
            run(["launchctl", "load", outfile], cwd=TARGET_DIR, user="root")
    print("All done.")


if __name__ == "__main__":
    # TODO: Make this impotent and reentrant safe
    if os.geteuid() == 0:
        post_install_two()
    else:
        post_install_one()



"""Offloading proxy.
This runs inside the main uWSGI process and sends offload headers
for each registered app. For unregistered apps, it displays a
web-based administration screen.

Must be run within the .flingdev folder in the user's home directory.
"""

import os
import json
from fling.start import start
from flask import Flask, request, render_template, redirect
from flask_bootstrap import Bootstrap5
from flask_caching import Cache
from . import __version__
from lastversion import has_update
from os import environ


config = None
with open("loophost.json") as appjson:
    config = json.loads(appjson.read())


admin = Flask(__name__)
bootstrap = Bootstrap5(admin)
cache = Cache(config={"CACHE_TYPE": "SimpleCache"})
cache.init_app(admin)
admin.register_blueprint(start, url_prefix="/start")  # unused but I need the templates


@cache.cached(timeout=3600)
def latest_version():
    return has_update(repo="fling-hub", at="pip", current_version=__version__)


@admin.context_processor
def inject_globals():
    return dict(NEW_VERSION=latest_version())


@admin.route("/unbind")
def unbind():
    project = request.args.get("project")
    if project and project in config["apps"]:
        del config["apps"][project]
        with open("loophost.json", "w") as appjson:
            appjson.write(json.dumps(config))
    return redirect("/")


@admin.route("/", defaults={"path": ""}, methods=["GET", "POST"])
@admin.route("/<path:path>", methods=["GET", "POST"])
def admin_page(path):
    fqdn = request.host.split(".")
    if len(fqdn) < 4:
        return render_template(
            "admin.html", apps=config["apps"], sharing={}, fqdn=".".join(fqdn)
        )
    project = fqdn.pop(0)
    if request.form.get("application_port"):
        config["apps"][project] = request.form.get("application_port")
        with open("loophost.json", "w") as appjson:
            appjson.write(json.dumps(config))
    return render_template(
        "local.html",
        apps=config["apps"],
        sharing={},
        project=project,
        fqdn=".".join(fqdn),
    )


if __name__ == "__main__":
    admin.run(host=f"unix://{os.getcwd()}/loophost.soc")
